import unittest, os, os.path, sys, urllib

import tornado.options
from tornado.options import options
from tornado.testing import AsyncHTTPTestCase
from torskel.torskel_app import TorskelServer
from tests.app_handlers import TestPage

APP_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(os.path.join(APP_ROOT, '..'))

# python -m tornado.test.runtests tests/test_handler.py


class TestHandlerBase(AsyncHTTPTestCase):
    def setUp(self):

        handlers = [
            (r"/", TestPage),

        ]

        self.app = TorskelServer(handlers)
        super(TestHandlerBase, self).setUp()

    def get_app(self):

        return self.app

    def get_http_port(self):
        return options.port


class TestBucketHandler(TestHandlerBase):
    def test_get_req(self):
        response = self.fetch(
            '/',
            method='GET',
            follow_redirects=False)

        self.assertEqual(response.code, 200)



import unittest
import re
import hashlib
from torskel.str_utils import get_hash_str
from torskel.str_utils import is_hash_str
from torskel.str_utils import valid_conversion
from torskel.str_utils import is_valid_ip
from torskel.str_utils import is_valid_mac

class TestStrUtils(unittest.TestCase):
    def setUp(self):
        self.hash_sha224_tmpl = re.compile(r"\b([a-f\d]{56}|[A-F\d]{56})\b")
        self.all_hash_tmpl = re.compile(r"^(?:[a-fA-F\d]{32,40})$|^(?:[a-fA-F\d]{52,60})$|^(?:[a-fA-F\d]{92,100})$")
        self.test_str = 'test me'
        self.test_hash_res = hashlib.sha224(self.test_str.encode('utf-8')).hexdigest()
        self.test_hash = '68df004a144aa21f31047e0635c246c990ea2a373c35cedfcac53966'

    def test_get_hash_str_tmpl(self):
        res = get_hash_str(self.test_str)
        assert self.hash_sha224_tmpl.match(res), True

    def test_get_hash_str(self):
        res = get_hash_str(self.test_str)
        assert res, self.test_hash_res

    def test_get_hash_str_wrong_type(self):
        res = get_hash_str([111])
        assert res is None

    def test_is_hash_str(self):
        res = is_hash_str(self.test_hash)
        self.assertEqual(res, True)

    def test_is_hash_str_wrong_val(self):
        res = is_hash_str(23423)
        self.assertEqual(res, False)

    def test_valid_ip(self):
        res = is_valid_ip('127.0.0.1')
        self.assertEqual(res, True)

    def test_valid_ip_wrong_val(self):
        res = is_valid_ip('327.0.0.1')
        self.assertEqual(res, False)

    def test_valid_mac(self):
        res = is_valid_mac('00:02:02:34:72:a5')
        self.assertEqual(res, True)

    def test_valid_mac_wrong_val(self):
        res = is_valid_mac('.1')
        self.assertEqual(res, False)


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    unittest.main(testRunner=runner)



import logging.handlers
import tornado.log

LOG_MSG_DEBUG_TMPL = '%s %s'


class TorskelLogMixin(object):

    def __init__(self):
        self.logger = tornado.log.gen_log
        self.log_msg_tmpl = LOG_MSG_DEBUG_TMPL

    # ################### #
    #  Logging functions  #
    # ################### #

    def get_log_msg(self, msg, grep_label=''):
        """
        Make message by template
        :param msg: message
        :param grep_label: label for grep
        :return: compiled message
        """
        try:
            res = self.log_msg_tmpl % (grep_label, msg)
        except:
            res = msg
        return res

    def log_debug(self, msg, grep_label=''):
        """
        Log debug message
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.debug(self.get_log_msg(msg, grep_label))

    def log_err(self, msg, grep_label=''):
        """
        Log error
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.error(self.get_log_msg(msg, grep_label))

    def log_exc(self, msg, grep_label=''):
        """
        Log exception
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.exception(self.get_log_msg(msg, grep_label))

    def set_mail_logging(self, mail_host, from_addr, to_addr, subject, credentials_list=None, log_level=logging.ERROR):
        """
        Init SMTP log handler for sendig log to email
        :param mail_host: host
        :param from_addr: from
        :param to_addr: to
        :param subject: subject
        :param credentials_list: (login, password)
        :param log_level: log level
        :return:
        """
        # TODO validate mail params try catch
        mail_logging = logging.handlers.SMTPHandler(mailhost=mail_host,
                                                    fromaddr=from_addr,
                                                    toaddrs=to_addr,
                                                    subject=subject,
                                                    credentials=credentials_list
                                                    )

        mail_logging.setLevel(log_level)
        self.logger.addHandler(mail_logging)


# -*- coding: utf-8 -*-
import json

try:
    from bson import json_util
except ImportError:
    json_util = False

import tornado.gen
from tornado.options import options
from torskel.str_utils import get_hash_str
from torskel.str_utils import is_hash_str
from torskel.torskel_mixins.log_mix import TorskelLogMixin


class TorskelHandler(tornado.web.RequestHandler, TorskelLogMixin):
    def __init__(self, application, request, **kwargs):
        super(TorskelHandler, self).__init__(application, request, **kwargs)
        # filter dict by list of keys
        self.filter_dict = lambda x, y: dict([(i, x[i]) for i in x if i in set(y)])


    def react_render(self, template_name, render_data_dict=None):
        if render_data_dict is None:
            render_data_dict = {}
        template = self.application.react_env.get_template(template_name)
        render_data_dict.update({'assets': self.application.react_assets})
        return self.write(template.render(render_data_dict))

    @staticmethod
    def get_hash_str(value, alg='sha224'):
        """
        Return hash by string
        :param value: source string
        :param alg: algorithm, default sha224
        :return: hash
        """
        return get_hash_str(value, alg)

    @staticmethod
    def is_hash_str(value):
        """
        Checks whether the hash string is a string of any algorithm
        :param value: hash string
        :return: boolean
        """
        return is_hash_str(value)

    def get_req_args(self, args_list=None):
        """
        Get list of params from request
        :param args_list: list of params
        :return: list of values
        """
        if not args_list:
            args_list = []
        return [self.get_argument(x, False) for x in args_list]

    def get_req_args_dict(self):
        """
        Return dict of request arguments
        :return: dict
        """

        try:
            res = {k: ''.join([i.decode('utf-8') for i in v]) for k, v in self.request.arguments.items()}
        except:
            self.log_exc('get_req_args_dict failed')
            res = {}
        return res

    def get_user_ip(self):
        """
        Return user ip from request object
        :return: user_ip
        """
        x_real_ip = self.request.headers.get("X-Real-IP")
        remote_ip = x_real_ip or self.request.remote_ip
        return remote_ip

    def _handle_request_exception(self, e):
        super(TorskelHandler, self)._handle_request_exception(e)
        msg = '''handler classname = %s \n
			   request = %s \n
		       exception = %s \n'''

        self.log_exc(msg % (type(self).__name__, self.request, e))



    async def http_request_get(self, url, **kwargs):
        """
        async http request. Method GET
        :param url: url
        :param from_json: boolean, convert response to dict
        :return: response
        """

        return await self.application.http_request_get(url, **kwargs)

    async def http_request_post(self, url, body, **kwargs):
        """
        async http request. Method POST
        :param url: url
        :param body: dict with POST-params
        :param from_json: boolean, convert response to dict
        :return: response
        """
        return await self.application.http_request_post(url, body, **kwargs)

    # TODO refact add params to kwargs
    async def set_redis_exp_val(self, key, val, exp, **kwargs):
        """
        Write value to redis
        :param key: key
        :param val: value
        :param exp: Expire time in seconds
        :param convert_to_json: bool
        :param use_json_utils: bool use json utils from bson
        """
        await self.application.set_redis_exp_val(key, val, exp, **kwargs)

    async def del_redis_val(self, key):
        """
        delete value from redis by key
        :param key: key

        """
        await self.application.del_redis_val(key)

    async def get_redis_val(self, key, **kwargs):
        """
        get value from redis by key
        :param key: key
        :param from_json: loads from json
        :param use_json_utils: bool use json utils from bson
        :return: value
        """
        res = await self.application.get_redis_val(key, **kwargs)
        return res



# -*- coding: utf-8 -*-
import json
import os.path
import logging.handlers
import tornado.log
import tornado.web
import xmltodict
import tornado.httpclient

try:
    from bson import json_util
except ImportError:
    json_util = False

try:
    import aioredis
except ImportError:
    aioredis = None

try:
    import aioodbc
except ImportError:
    aioodbc = None

try:
    from jinja2 import Environment, FileSystemLoader
    jinja2_import = True
except ImportError:
    jinja2_import = False

try:
    import pycurl
except ImportError:
    pycurl = None


from tornado.options import options
from urllib.parse import urlencode
from tornado.httpclient import AsyncHTTPClient
from tornado.autoreload import watch


settings = {
    # 'cookie_secret': options.secret_key,
    # 'xsrf_cookies': True,
}

options.define('debug', default=True, help='debug mode', type=bool)
options.define("port", default=8888, help="run on the given port", type=int)

# http-client params
options.define("max_http_clients", default=100, type=int)
options.define("http_client_timeout", default=30, type=int)
options.define("use_curl_http_client", default=False, type=bool)

# mail logger params
options.define('use_mail_logging', default=False, help='SMTP log handler', type=bool)
options.define("log_mail_subj", default='', type=str)
options.define("log_mail_from", default='', type=str)
options.define("log_mail_to", default=[], type=list)
options.define("log_mail_host", default='', type=str)
options.define("log_mail_user", default='', type=str)
options.define("log_mail_psw", default='', type=str)

# redis params
options.define('use_redis', default=False, help='use redis', type=bool)
options.define('use_redis_socket', default=True, help='connection to redis unixsocket file', type=bool)
options.define("redis_min_con", default=5, type=int)
options.define("redis_max_con", default=10, type=int)
options.define("redis_host", default='127.0.0.1', type=str)
options.define("redis_port", default=6379, type=int)
options.define("redis_socket", default='/var/run/redis/redis.sock', type=str)
options.define("redis_psw", default='', type=str)
options.define("redis_db", default=-1, type=int)

# reactjs params
options.define('use_reactjs', default=False, help='use reactjs', type=bool)
options.define("react_assets_file", default='webpack-assets.json', type=str)


class TorskelServer(tornado.web.Application):
    def __init__(self, handlers, root_dir=None, static_path=None, template_path=None,
                 create_http_client=True, **settings):

        # TODO add valiate paths
        if root_dir is not None:
            app_static_dir = os.path.join(root_dir, "static") if static_path is None else static_path

            app_template_dir = os.path.join(root_dir, "templates") if template_path is None else template_path
        else:
            app_template_dir = app_static_dir = None

        super().__init__(handlers, static_path=app_static_dir,
                         template_path=app_template_dir,
                         **settings)
        #self.redis_connection_pool = None
        self.logger = tornado.log.gen_log

        tornado.ioloop.IOLoop.configure('tornado.platform.asyncio.AsyncIOMainLoop')

        if options.use_mail_logging:
            if options.log_mail_user == '' and options.log_mail_psw == '':
                credentials_list = None
            else:
                credentials_list = [options.log_mail_user, options.log_mail_psw]

            self.set_mail_logging(options.log_mail_host, options.log_mail_from, options.log_mail_to,
                                  options.log_mail_subj, credentials_list)

        if options.use_reactjs:
            if not jinja2_import:
                self.react_env = self.react_assets = None
                raise ImportError('Required package jinja2 is missing')
            else:
                self.react_env = Environment(loader=FileSystemLoader('templates'))
                self.react_assets = self.load_react_assets()

        else:
            self.react_env = self.react_assets = None

        self.http_client = AsyncHTTPClient(max_clients=options.max_http_clients) if create_http_client else None
        if options.use_curl_http_client:
            self.log_debug(options.use_curl_http_client, grep_label='use_curl_http_client')
            if pycurl is None:
                raise ImportError('Required package for pycurl CurlAsyncHTTPClient  is missing')
            else:
                self.log_debug('configure curl')
                AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")

    # ########################### #
    #  Validate params functions  #
    # ########################### #
    @staticmethod
    def validate_options():
        return True

    @staticmethod
    def validate_path():
        return True

    @staticmethod
    def validate_mail_params():
        return True

    # ######################## #
    #  ReactJS render support  #
    # ######################## #

    @staticmethod
    def load_react_assets():
        try:
            fn = options.react_assets_file
            with open(fn) as f:
                watch(fn)
                assets = json.load(f)
        except IOError:
            assets = None
        except KeyError:
            assets = None
        return assets

    # ################# #
    #  Init with loop   #
    # ################# #

    def init_with_loop(self, loop=None):
        if options.use_redis:
            # check aioredis lib
            if aioredis is None:
                raise ImportError('Required package aioredis is missing')
            else:
                self.init_redis_pool(loop)



    # ############################# #
    #  Async Http-client functions  #
    # ############################# #
    async def http_request_post(self, url, body, **kwargs):
        """
        http request. Method POST
        :param url: url
        :param body: dict with POST-params
        :param from_json: boolean, convert response to dict
        :return: response
        """
        from_json = kwargs.get('from_json', False)
        from_xml = kwargs.get('from_xml', False)
        log_timeout_exc = kwargs.get('log_timeout_exc', True)
        self.log_debug(log_timeout_exc, grep_label='log_timeout_exc')
        res = None
        try:
            headers = None
            param_s = urlencode(body)

            res_fetch = await self.http_client.fetch(url, method='POST', body=param_s, headers=headers)

            res_s = res_fetch.body.decode(encoding="utf-8") if res_fetch is not None else res_fetch

            res = res_s
            if from_json:
                res = json.loads(res_s)
            if from_xml:
                res = json.loads(json.dumps(xmltodict.parse(res_s)))
        except tornado.httpclient.HTTPError as e:
            if e.code == 599:
                if log_timeout_exc is True:
                    self.log_exc('http_request_get failed by timeout url = %s' % url)
                else:
                    self.log_debug('http_request_get failed by timeout')
                res = None

        except Exception:
            res = None
            self.log_exc('http_request_post failed! url = %s  body=%s ' % (url, body))

        return res

    async def http_request_get(self, url, **kwargs):
        """
        http request. Method GET
        :param url: url
        :param from_json: boolean, convert response to dict
        :return: response
        """
        from_json = kwargs.get('from_json', False)
        from_xml = kwargs.get('from_xml', False)
        log_timeout_exc = kwargs.get('log_timeout_exc', True)
        self.log_debug(log_timeout_exc, grep_label='log_timeout_exc')
        res = None
        try:
            res_fetch = await self.http_client.fetch(url)
            res_s = res_fetch.body.decode(encoding="utf-8") if res_fetch is not None else res_fetch

            res = res_s
            if from_json:
                res = json.loads(res_s)

            if from_xml:
                res = json.loads(json.dumps(xmltodict.parse(res_s)))
        except tornado.httpclient.HTTPError as e:
            if e.code == 599:
                if log_timeout_exc is True:
                    self.log_exc('http_request_get failed by timeout url = %s' % url)
                else:
                    self.log_debug('http_request_get failed by timeout')
                res = None
        except Exception:
            self.log_exc('http_request_get failed! url = %s' % url)
            res = None


        return res

    # ######################## #
    #  Redis client functions  #
    # ######################## #

    def init_redis_pool(self, loop=None):
        """
        Init redis connection pool
        :param loop: ioloop

        """
        redis_addr = options.redis_socket if options.use_redis_socket else (options.redis_host, options.redis_port)
        # TODO validate redis connection params
        redis_psw = options.redis_psw

        if redis_psw == "":
            redis_psw = None

        redis_db = options.redis_db
        if redis_db == -1:
            redis_db = None

        self.redis_connection_pool = loop.run_until_complete(aioredis.create_pool(
            redis_addr, password=redis_psw, db=redis_db,
            minsize=options.redis_min_con, maxsize=options.redis_max_con))

    async def set_redis_exp_val(self, key, val, exp=None, **kwargs):
        """
        Write value to redis
        :param key: key
        :param val: value
        :param exp: Expire time in seconds
        :param convert_to_json: bool
        :param use_json_utils: bool use json utils from bson

        """
        convert_to_json = kwargs.get('convert_to_json', False)
        use_json_utils = kwargs.get('use_json_utils', False)
        if convert_to_json:
            if use_json_utils and json_util:
                val = json.dumps(val, default=json_util.default)
            else:
                val = json.dumps(val)

        await self.redis_connection_pool.execute('set', key, val)
        if isinstance(exp, int):
            await self.redis_connection_pool.execute('expire', key, exp)

    async def del_redis_val(self, key):
        """
        delete value from redis by key
        :param key: key

        """

        await self.redis_connection_pool.execute('del', key)

    async def get_redis_val(self, key, **kwargs):
        """
        get value from redis by key
        :param key: key
        :param from_json: loads from json
        :param use_json_utils: bool use json utils from bson
        :return: value
        """
        try:
            from_json = kwargs.get('from_json', False)
            use_json_utils = kwargs.get('use_json_utils', False)

            r = self.redis_connection_pool.execute('get', key)
            redis_val = r.decode('utf-8') if r is not None else r
            if redis_val:
                if use_json_utils and json_util:
                    res = json.loads(redis_val, object_hook=json_util.object_hook) if from_json else redis_val
                else:
                    res = json.loads(redis_val) if from_json else redis_val
            else:
                res = None
            return res
        except:
            self.log_exc('get_redis_val failed key = %s' % key)
            res = None
            return res

    # ################### #
    #  Logging functions  #
    # ################### #

    def get_log_msg(self, msg, grep_label=''):
        """
        Make message by template
        :param msg: message
        :param grep_label: label for grep
        :return: compiled message
        """
        try:
            res = self.log_msg_tmpl % (grep_label, msg)
        except:
            res = msg
        return res

    def log_debug(self, msg, grep_label=''):
        """
        Log debug message
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.debug(self.get_log_msg(msg, grep_label))

    def log_err(self, msg, grep_label=''):
        """
        Log error
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.error(self.get_log_msg(msg, grep_label))

    def log_exc(self, msg, grep_label=''):
        """
        Log exception
        :param msg: message
        :param grep_label: label for grep
        :return:
        """
        self.logger.exception(self.get_log_msg(msg, grep_label))

    def set_mail_logging(self, mail_host, from_addr, to_addr, subject, credentials_list=None,
                         log_level=logging.ERROR):
        """
        Init SMTP log handler for sendig log to email
        :param mail_host: host
        :param from_addr: from
        :param to_addr: to
        :param subject: subject
        :param credentials_list: (login, password)
        :param log_level: log level
        :return:
        """
        # TODO validate mail params try catch
        mail_logging = logging.handlers.SMTPHandler(mailhost=mail_host,
                                                    fromaddr=from_addr,
                                                    toaddrs=to_addr,
                                                    subject=subject,
                                                    credentials=credentials_list
                                                    )

        mail_logging.setLevel(log_level)
        self.logger.addHandler(mail_logging)



# -*- coding: utf-8 -*-
import hashlib
import re
import pickle
import ipaddress

hash_sha224_tmpl = re.compile(r"\b([a-f\d]{56}|[A-F\d]{56})\b")
all_hash_tmpl = re.compile(r"^(?:[a-fA-F\d]{32,40})$|^(?:[a-fA-F\d]{52,60})$|^(?:[a-fA-F\d]{92,100})$")
mac_address = re.compile('^' + '[\:\-]'.join(['([0-9a-f]{2})'] * 6) + '$')


def is_valid_ip(ip):
    """
    Validate ip address
    :param ip: ip address
    :return: boolean
    """
    res = False
    if isinstance(ip, str):
        try:
            ipaddress.ip_address(ip)
            res = True
        except ValueError:
            res = False
    return res


def is_valid_mac(mac):
    """
    Validate mac address
    :param mac:
    :return: boolean
    """
    res = False
    if isinstance(mac, str):
        if len(mac) > 0:
            try:
                res = mac_address.match(mac.lower()) is not None
            except Exception:
                res = False
    return res


def chr_set_null(chr_value):
    """sql util function"""
    return "null" if chr_value is None or not chr_value else ''.join(("'", chr_value, "'"))


def int_set_null(int_value):
    """sql util function"""
    return "null" if int_value is None else str(int_value)


def valid_conversion(val, type_to_convert):
    """
    Checking whether it is possible to convert val to the specified type
    :param val: value
    :param type_to_convert: type
    :return: boolean
    """

    if isinstance(type_to_convert, type):
        try:
            val = type_to_convert(val)
            res = True
        except ValueError:
            res = False

    else:
        raise TypeError
    return res


def get_hash_str(value, alg='sha224'):
    """
    Return hash from string
    :param value: string
    :param alg: algorithm, default sha224
    :return: hash
    """
    res = None

    try:
        if isinstance(value, str):
            res = getattr(hashlib, alg)(value.encode('utf-8')).hexdigest()
    except:
        pass

    return res


def is_hash_str(value):
    """
    Checks whether the string is a hash of any algorithm
    :param value:
    :return: boolean
    """
    res = False
    if isinstance(value, str):
        if len(value) > 0:
            try:
                res = all_hash_tmpl.match(value) is not None
            except Exception:
                res = False
    return res


def pack_iters(val):
    """
    Packs the iterated type into a string
    :param val:
    :return: str
    """

    try:
        if isinstance(val, (dict, list, tuple, enumerate)):
            res = pickle.dumps(val)
        else:
            res = val
    except pickle.PicklingError:
        res = None

    return res


def unpack_iters(val):
    """
    Extracts a string into an iterated type
    :param val:
    :return: iter
    """

    try:
        res = pickle.loads(val)
    except pickle.UnpicklingError:
        res = None
    return res
